// s2250demo.c							
// Copyright 2006-2007 Sensoray co. Inc.
// Uses open source libraries ffmpeg, lame

/*
Copyright (C) 2003  Sensoray Co., Inc.  This program is free software;
you can redistribute it and/or modify it under the terms of the GNU
General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

You must obey the GNU General Public License in all respects for all of
the code used. If you modify this file, you may extend this exception
to your version of the file, but you are not obligated to do so. If you
do not wish to do so, delete this exception statement from your version.
*/

// The necessary headers /////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <pthread.h>
#include <sched.h>
#include <sys/mman.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <assert.h>
#include "s2250mid.h"
#include "wavfmt.h" // in ../../Include


static int G_polling = 0;

#define AUDIOBUFSIZE    ( (48000 * 4 / 30) + 300)  


#define ENABLED				1
#define DISABLED			0

#define TRUE				1
#define FALSE				0
/* Maximum length of user command string. */
#define MAXLEN				80
/* Maximum number of boards supported by the driver. */
#define MAXNUMBOARDS	    MAX_BOARDS
// Stream mode:
				
#define MODE_IDLE             0	// Halted.
#define MODE_RECORD_VIDONLY   1 // Streaming to file(raw video)
#define MODE_RECORD_AUDONLY   2 // Streaming to file(audio only)
#define MODE_RECORD_SPLIT     4 // Streaming to file( raw video one file, audio another file)
#define MODE_STREAMONLY       5


static int G_numboards=0;
// Number of MPEG bytes to transfer in a single chunk
#define MPEG_BUF_SIZE	(800*576*3)
static char G_buffer[MPEG_BUF_SIZE]; // global buffer for snapshots

// board interface

/* storage */
typedef struct BOARD {
	int handle; // board instance
	int mode;   //  Stream mode.
	int stop_rq;   //  Stream Halt requested.
	char fname[MAXLEN];  //  File name for data.
	pthread_t vid_thread_id; //  Handle to stream i/o vid thread.
	pthread_t aud_thread_id; //  Handle to stream i/o vid thread.
	unsigned int mpeg_count; //  Number of MPEG bytes transferred.
	unsigned int aud_count;	 //  Number of audio bytes transferred.
	int          vidsys;
	MIDSRAY_ENCODING          encode;
	int                       bitrate;
	int                       source;
	int                       size;
	int                       iframesonly;
	int                       hue;
	int                       vsat;
	int                       contrast;
	int                       brightness;
	int                       xstart; // caption
	int                       ystart; // caption
	char                      text[80];
} BOARD;

static struct BOARD bd[MAX_BOARDS];// Board objects.
static int act_bdnum = 0; // Board 0 is active by default.

static int app_init( void );
static void app_shutdown( void );
static void cmd_processor( void );
static void *vid_thread_func( void* arg );
static void *aud_thread_func( void* arg );
static void start_stream( struct BOARD *b, char *fname, int new_stream_mode );
static void stop_stream( struct BOARD *b );

static int G_stopping = 0;

/*
 * Init the system and execute user commands until termination.
 */
int main( int argc, char *argv[] )
{
  int rtnval;
  // Attempt to open and initialize all boards.
  rtnval = app_init();

  // If all boards successfully opened and initialized ...
  if ( rtnval == 0 )
  {
    // Execute user commands until Application Quit command is invoked.
    cmd_processor();
  } else {
    fprintf(stderr, "Failed to open board\n");
  }

  // Close all opened boards.
  app_shutdown();

  // Exit application.
  return 0;
}


/*
 * Open and initialize one board.  Returns: 0=success, -1=error.
 */
static int board_open( int board_num )
{
  // Set pointer to board's object.
  struct BOARD *b = &bd[board_num];
  int res;
  // Initialize object.
  b->mode			= MODE_IDLE;
  b->fname[0]	= 0;
  // device parameters
  b->vidsys = MIDSRAY_VIDSYS_NTSC;
  b->size = 0;          // full size
  b->bitrate = 3000000; // 3MBit/s
  b->encode = MIDSRAY_ENCODE_MPEG1;
  b->source = 0;        // 
  b->iframesonly = 0;
  b->xstart = 0;
  b->ystart = 20;

  //SN_Debug(1);

  // open board
  res = SN_OpenBoard( board_num);

  if (res != 0) {
      printf("board open failed %d\n", board_num);
      return -1;
  }
    b->handle = board_num;
    // optional set blocking or polling
    if (G_polling) {
	    SN_SetPolling(board_num);
    } else {
	    SN_SetBlocking(board_num);
    }

    b->vid_thread_id = 0;
    b->aud_thread_id = 0;
    SN_SetBasicChipSettings( board_num, b->encode, b->size,
			     b->vidsys, b->bitrate, b->iframesonly);
    return 0;
}




/* enumerate, open and initialize all boards.
 * Returns: -1=error, 0=success.
*/
static int app_init( void )
{
  int		i;
  // For each board ...
  G_numboards = SN_EnumBoards();

  for ( i = 0; i < G_numboards; i++ )
  {
    // Specify board's device name.

    // Open device.  Return error if open failed.
    if ( board_open( i ) == -1 )
	    break;
  }
  if ( i == 0)
	  return -1;
  G_numboards = i;
  return 0;
}

/* terminate all streams and close all boards
 */
static void app_shutdown( void )
{
  struct BOARD *b;

  // For each board ...
  for (b = bd; b < &bd[G_numboards]; b++) {
    // Terminate the board's I/O thread if it exists.
    stop_stream( b );

    // Close the board.
    printf("close board\n");
    SN_CloseBoard( b->handle );
  }

  printf( "\n" );
}

//////////////////////////////////////////////////////////////
// Display the user prompt.

static void prompt( void )
{
  printf( "2250demo[%d]>", act_bdnum );
  fflush( stdout );
}


//////////////////////////////////////////////////////////////
// Display the command menu.

static void show_menu( void )
{
  printf( "+-S2250DEMO-COMMANDS-----------------------------------------------------------+\n" );
  printf( "|   board n ....... select board: 0..7                                         |\n" );
  printf( "|   vinput n ....... set video source: 0=S-Video, 1=Composite                  |\n" );
  printf( "|   ainput n ....... set audio source: 0=line, 1=mic, 2=mic with boost         |\n" );
  printf( "|   h[elp] ........ display this list                                          |\n" );
  printf( "|   q[uit] ........ quit program                                               |\n" );
  //printf( "+-AUDIO------------------------------------------------------------------------+\n" );
  printf( "+-VIDEO------------------------------------------------------------------------+\n" );
  printf( "|   bright n ...... set luminance brightness: 0..100                           |\n" );
  printf( "|   contrast n .... set luminance contrast: 0..100                             |\n" );
  printf( "|   vsat n ........ set chrominance saturation: 0..100                         |\n" );
  printf( "|   vhue n  ....... set hue -50..50(applies to NTSC only)                      |\n" );
  printf( "|   vsystem n ..... set video system: 0=NTSC, 1=PAL                            |\n" );
  //printf( "+-PREVIEW----------------------------------------------------------------------+\n" );
  //printf( "|   fifo   ........ 0 fifo off, 1--fifo on (needs xine installed)              |\n");
  //printf( "|   vstatus ....... get status of video input                                  |\n" );
  printf( "+-COMPRESSION------------------------------------------------------------------+\n" );
  printf( "|   encode n ...... set encoding type:                                         |\n" );
  printf( "|                     0=MPEG1, 1=mpeg2, 2=mpeg4(divx use mplayer to play)      |\n" );
  printf( "|   size n ........ set size 0==normal, 1==half size:                          |\n" );
  printf( "|   vrate n ....... set video bits/second: 100000..10000000                    |\n" );
  printf( "+-STREAM-----------------------------------------------------------------------+\n" );
  printf( "|   recvid fname .. record video stream to MPEG file; def name is \"tmp.mpg\"    |\n" );
  printf( "|   recaud fname .. record audio stream to wav file; def name is \"tmp.wav\"     |\n" );
  printf( "|   recsplt fname . Record Video stream and Audio stream separately            |\n" );
  printf( "|   stop .......... stop recording and close file                              |\n" );
  printf( "+-CAPTION----------------------------------------------------------------------+\n" );
  printf( "|   xstart n ...... set window position for x_start: 8..700                    |\n" );
  printf( "|   ystart n ...... set window position for y_start: 0..478 / 574 (NTSC/PAL)   |\n" );
  printf( "|   text string ... set text string for overlay:  any ascii string             |\n" );
  printf( "|   caption n ..... set caption off/on: 0=off, 1=on (after stream started)     |\n" );
  printf( "+------------------------------------------------------------------------------+\n" );
}


///////////////////////////////////////////////////////////////////////////////
// Decode and execute user command.  Returns TRUE if Quit selected.

static int exec_command( char *cmd, char *arg )
{
  struct BOARD	*b;

  // Convert argument to integer value (not relevant for all commands).
  int arg_val = -1;
  sscanf( arg, "%d", &arg_val );

  // Decode and execute command -----------------------------------

  // Terminate application.
  if ( cmd[0] == 'q' )
    return TRUE;

  // Select board.
  if	( strcmp( cmd, "board" ) == 0 )
  {
    if ( ( arg_val >= 0 ) && ( arg_val < G_numboards ) )
      act_bdnum = arg_val;
    else
      printf( "Illegal or missing argument; must be in range 0..%d\n", G_numboards-1 );
    prompt();
    return FALSE;
  }

  // Show help.
  if ( ( cmd[0] == 'h' ) || ( cmd[0] == '?' ) )
  {
    show_menu();
    prompt();
    return FALSE;
  }

  // Prompt for user command.
  prompt();

  // Set pointer to active board's object.
  b = &bd[act_bdnum];

  if( strcmp( cmd, "vinput"    ) == 0 )
  {
      b->source = arg_val;
      SN_SetVideoSource( act_bdnum, arg_val);
  }
  else if ( strcmp( cmd, "ainput"    ) == 0 )
  {
      b->source = arg_val;
      SN_SetAudioSource( act_bdnum, arg_val);
  }
  else if ( strcmp( cmd, "contrast"  ) == 0 )
  {
      b->contrast = arg_val;
      SN_SetContrast(act_bdnum, arg_val);
  }
  else if ( strcmp( cmd, "vhue"  ) == 0 )
  {
	  if (b->vidsys != MIDSRAY_VIDSYS_NTSC) {
		  printf("Hue setting applies to NTSC only\n");
		  prompt();
		  return FALSE;
	  }
	  
     b->hue = arg_val;
     SN_SetHue( act_bdnum, arg_val);
  }
  else if ( strcmp( cmd, "bright"  ) == 0 )
  {
      b->brightness = arg_val;
      SN_SetBrightness( act_bdnum, arg_val);
  }
  else if ( strcmp( cmd, "vsat"  ) == 0 )
  {
      b->vsat = arg_val;

      SN_SetSaturation( act_bdnum, arg_val);
  }
  else if ( strcmp( cmd, "vsystem"   ) == 0 )
  {
      if( arg_val == 0)
      {
          printf("setting NTSC\n");   
        b->vidsys = MIDSRAY_VIDSYS_NTSC;
      }
      else
      {
        printf("setting PAL\n");   
        b->vidsys = MIDSRAY_VIDSYS_PAL;
      }
      SN_SetVideoSystem(act_bdnum, b->vidsys);
      SN_SetBasicChipSettings( act_bdnum, b->encode,b->size , b->vidsys, b->bitrate, b->iframesonly);
      prompt();
  }
  else if ( strcmp( cmd, "encode"      ) == 0 )
  {
	  MIDSRAY_ENCODING fmt = MIDSRAY_ENCODE_MPEG1;
        switch( arg_val)
        { 
        case 0: // MPEG1
            fmt = MIDSRAY_ENCODE_MPEG1;
            break;
        case 1: // MPEG2
            fmt = MIDSRAY_ENCODE_MPEG2;
            break;
        case 2:
            fmt = MIDSRAY_ENCODE_MPEG4DIVX;
            break;
	default:
		printf("invalid encode value\n");
		prompt();
		return 0;
        }
        SN_SetBasicChipSettings( act_bdnum, fmt, b->size , b->vidsys, b->bitrate, b->iframesonly);
        b->encode = fmt;
        printf("Changed Encoding(restart stream if running)\n");
        printf("Imporant: Changing encoding resets brightness,\n");
	printf("contrast, vhue(NTSC), and saturation.\n");
        prompt();
        return 0;
  }
  else if ( strcmp( cmd, "vrate"     ) == 0 )
  {
      b->bitrate = arg_val;
      printf("setting new bitrate %d\n", b->bitrate);
      prompt();
      SN_SetBasicChipSettings( act_bdnum, b->encode, b->size , b->vidsys, b->bitrate, b->iframesonly);
      SN_SetBitrate(act_bdnum, b->bitrate);
  }
  else if ( strcmp( cmd, "size"     ) == 0 )
  {
      if( arg_val == 0)
      {
          printf("using full size\n");
          b->size = 0;
      }
      else
      {
          printf("using half size\n");
	  if (b->bitrate > 1900000) {
	    printf("bitrate too high for half size, lowering bitrate to 1.9M\n");
	    b->bitrate = 1900000;
	  }
          b->size = 1;
      }
      prompt();
      SN_SetBasicChipSettings( act_bdnum, b->encode,b->size , b->vidsys, b->bitrate, b->iframesonly);
  }
  else if ( strcmp( cmd, "snapshot"  ) == 0 )
  {
	  int        rt;
	  int        len;
	  frame_info_t fi;
	  len = MPEG_BUF_SIZE;
      
	  if( b->mode != MODE_STREAMONLY)
	  {
		  printf("invalid mode for snapshot in demo(use stream to start)\n");
		  prompt();
		  return FALSE;
	  }
	  if( b->encode != MIDSRAY_ENCODE_MOTIONJPEG)
	  {
		  printf("snapshot supported for MOTIONJPEG only\n");
		  prompt();
		  return FALSE;
	  }
      
	  rt = SN_GetOneFrame( b->handle, (unsigned char *) G_buffer, len, &fi);

	  if( rt == 0 && (b->encode == MIDSRAY_ENCODE_MOTIONJPEG))
	  {
		  FILE *file;
		  file = fopen( arg, "wb");
		  fwrite( G_buffer, 1, fi.data_length, file );
		  fclose(file);
		  printf("wrote %d bytes to %s\n", fi.data_length, arg);
	  }
	  else
	  {
		  printf("get snapshot failed(fail or not MOTIONJPEG\n");
	  }
  }
  else if ( strcmp( cmd, "stream"    ) == 0 )
  {
	  SN_StartStream( act_bdnum);
	  b->mode = MODE_STREAMONLY;
  }
  else if ( strcmp( cmd, "recvid"    ) == 0 )	start_stream( b, ( arg[0] == 0 ) ? "tmp.mpg" : arg, MODE_RECORD_VIDONLY );
  else if ( strcmp( cmd, "recaud"    ) == 0 )	start_stream( b, ( arg[0] == 0 ) ? "tmp.wav" : arg, MODE_RECORD_AUDONLY );
  else if ( strcmp( cmd, "recsplt"   ) == 0 )	start_stream( b, ( arg[0] == 0 ) ? "tmp" : arg, MODE_RECORD_SPLIT );
  else if ( strcmp( cmd, "stop"      ) == 0 )	stop_stream( b );
  else if ( strcmp( cmd, "xstart"    ) == 0 )	
  {
	  b->xstart = arg_val;
  }
  else if ( strcmp( cmd, "ystart"    ) == 0 ) {
	  b->ystart = arg_val;
  } else if ( strcmp( cmd, "text"      ) == 0 )  {
	  strncpy( b->text, arg, 80);
  } else if ( strcmp( cmd, "caption"   ) == 0 )  {
	  if (b->mode == MODE_IDLE) {
		  printf("Caption must be turned on after stream started\n");
		  prompt();
		  return FALSE;
	  }
	  SN_SetCaption( act_bdnum, b->xstart, b->ystart, b->text);
	  printf("Caption set\n");
	  prompt();
  }
  else if ( cmd[0] != 0 )
  {
	  printf( "Invalid command: %s\n", cmd );
	  prompt();
  }

  return FALSE;
}

////////////////////////////////////////////////////////////////////
// Execute user commands until Application Quit is requested.


static void cmd_processor( void )
{
	char					cmdbuf[MAXLEN];
	char					cmd[MAXLEN];
	char					arg[MAXLEN];
	char					*c;

	// Display the command menu.
	show_menu();

	// Prompt for user command.
	prompt();

	// Repeat until Application Terminate is requested ...
	for ( ;G_stopping == 0 ; )
	{
		// Input user command string, and if no errors encountered ...
		if ( fgets( cmdbuf, MAXLEN - 1, stdin ) != NULL )
		{
			// Truncate newline char at end of command string.
			cmdbuf[ strlen( cmdbuf ) - 1 ] = 0;

			// If command line is not blank ...
			if ( cmdbuf[0] != 0 )
			{
				// Force command string to lower case, except command=="text".
				if ( strncmp( cmdbuf, "text", 4 ) != 0 )
					for ( c = cmdbuf; *c != 0; c++ ) 
						*c = tolower( *c );

				// Parse command string into opcode and argument.
				cmd[0] = arg[0] = 0;
				sscanf( cmdbuf, "%s %s", cmd, arg );

				// Decode and execute command
				if ( exec_command( cmd, arg ) )
					break;

			}
			else
			{
				// Prompt for user command.
				prompt();
			}
		}
	}
}




// Start MPEG data streaming to/from a file.
static void start_stream( struct BOARD *b, char *fname, int new_stream_mode )
{
	pthread_attr_t attr;

	// If we are already streaming ...
	if ( b->mode != MODE_IDLE )
	{
		// Abort command.
		printf( "already streaming\n" );
		prompt();
		return;
	}

	// Make the MPEG filename visible to the I/O thread.
	strcpy( b->fname, fname );

	// Indicate the new stream mode.
	b->mode = new_stream_mode;

	// Clear the stream thread's termination request.
	b->stop_rq = 0;

	// Create detached thread that will manage the MPEG stream flow.
	pthread_attr_init( &attr );
	{
		struct sched_param param;
		pthread_attr_getschedparam(&attr, &param);
		param.sched_priority = 10;
		pthread_attr_setschedparam(&attr, &param);
	}
	pthread_attr_setdetachstate( &attr, PTHREAD_CREATE_JOINABLE );

	pthread_create( &b->vid_thread_id, &attr, &vid_thread_func, b );
	pthread_create( &b->aud_thread_id, &attr, &aud_thread_func, b );
	pthread_attr_destroy( &attr );
}

//////////////////////////////////////////////////
// Halt MPEG data stream to/from a file.

static void stop_stream( struct BOARD *b )
{

	if( b->mode == MODE_STREAMONLY)
	{
		// no thread associated with MODE_STREAMONLY
		SN_StopStream(b->handle);
		b->mode = MODE_IDLE;
	}

	// If stream i/o thread is running ...
	if ( b->mode != MODE_IDLE )
	{
		// Request thread termination.
		b->stop_rq = 1;
		if( b->vid_thread_id) 
			pthread_join( b->vid_thread_id,NULL);
		if( b->aud_thread_id) 
			pthread_join( b->aud_thread_id,NULL);
		b->mode = MODE_IDLE;
	}
	printf("Stream stopped successfully\n");
	prompt();

}


//  MPEG STREAM
// Video Elementary Stream.
// Get data from chip without FFMPEG libraries or muxing
// Copy a block of MPEG data from board to file.
// Returns number of bytes copied.
static int mpeg_to_file( struct BOARD *b, FILE *file )
{
	char	mpeg_buf[MPEG_BUF_SIZE];
	int		len;
	int rt;
	frame_info_t fi;;
	static char num = 0;
	num++;
	len = MPEG_BUF_SIZE;
  
//  memset(mpeg_buf,0xee,MPEG_BUF_SIZE);  // debug only

	rt = SN_GetOneFrame( b->handle, (unsigned char *) mpeg_buf, len, &fi);

	if (rt != 0) {
		if (G_polling) {
	  
			// device not ready, wait and try again later
			//  printf("rt %d  %d %d\n", rt, fi.data_length,0);//fi.FrameType); 
			usleep(10000); 
			return 0;
		}
		return 0;
	}
	//printf("got this much data %d\n", fi.data_length);
	len = fi.data_length;
	b->mpeg_count += len;
	// Write raw MPEG data to file.
	fwrite( mpeg_buf, 1, len, file );
	// Return number of bytes transferred.
	return len;
}

/////////////////////////////////////////////////////
// Copy a block of Audio data from board to file.
// Returns number of bytes copied.
static int aud_to_file( struct BOARD *b, FILE *file )
{
	char	    aud_buf[AUDIOBUFSIZE];
	int		len;
	int       rt;
	int sec,usec;
	static char num = 0;
	num++;
	// use 48000 samplerate frame size, 16bits
	len = 1152 * 4;
	assert( len + 8 < AUDIOBUFSIZE);  // +8 for timestamp

	rt = SN_GetAudioDataPlusTime(b->handle, (unsigned char *) aud_buf, &len,&sec,&usec);
	// optional print the timestamps
	//printf("len %d sec %d usec %d\n", len,sec,usec);
	if( rt != 0) {
		// device not ready, wait and try again later
		//printf("rt %d  %d %d\n", rt, fi.data_length,fi.FrameType); 
		if (G_polling) {
			usleep(10000); 
		}
		return 0;
	}

	assert( len > 0);
	b->aud_count += len;
	// Write raw audio data to file.
	fwrite( aud_buf, 1, len, file );
	// Return number of bytes transferred.
	return len;
}




/* video THREAD MAIN.  process video data
 * (in record mode, write raw MPEG data until stream is stopped.)
 * The arg is a pointer to the device's BOARD object.
 */
void *vid_thread_func( void *arg )
{
	FILE *file;  // MPEG data file.
	unsigned int rlen;// MPEG bytes transferred.
	time_t starttime; //Stream start time.
	BOARD *b = (BOARD *)arg;// Cache pointer to board object.
	char fname[260];
	// Select the function that will be used to move MPEG data
	// between file and board.
	int (* mpeg_transfer)( struct BOARD *b, FILE *file ) = mpeg_to_file;
	// Initialize stream statistics.
	b->mpeg_count = 0;		// No MPEG bytes have been transferred.

	time( &starttime );		// Log the start time.
  
	if( b->mode == MODE_RECORD_AUDONLY)
	{
		// end this thread, recording audio only
		b->vid_thread_id = 0;
		return 0;
	}
	else if( b->mode == MODE_RECORD_VIDONLY)
	{
		mpeg_transfer = mpeg_to_file;
		strcpy( fname, b->fname);
	}
	else if( b->mode == MODE_RECORD_SPLIT)
	{
		mpeg_transfer = mpeg_to_file;
		strcpy( fname, b->fname);
		strcat( fname, ".mpg");
	}

	if(( b->mode == MODE_RECORD_VIDONLY) || ( b->mode == MODE_RECORD_SPLIT))
	{
		// Attempt to open the MPEG data file.  If successful ...
		if ( ( file = fopen(fname, "wb" ) ) != NULL )
		{
			// Report status.
			printf( "Starting stream\n");
			SN_StartStream( b->handle);
			prompt();
			// Until stream termination is requested ...
			while ( !b->stop_rq )
			{
				rlen = mpeg_transfer( b, file);
				// Allow other processes to execute if we are in non-blocking mode and no data was available.
				if ( ( rlen == 0 ) && G_polling )
				{
					// Sleep for approximately 10 milliseconds.
					struct timespec tv;
					tv.tv_sec = (time_t)0;
					tv.tv_nsec = (long)( 0.01 * 1e+9 );
					nanosleep( &tv, &tv );
				}
			}
			// Halt the stream.
			printf("calling stop stream\n");
			if( b->mode == MODE_RECORD_SPLIT) {
				// must give time for audio thread to exit
				// before calling SN_StopStream
				if( b->aud_thread_id) {
					pthread_join( b->aud_thread_id,NULL);
				}

			}
			SN_StopStream( b->handle);
			// Close the MPEG data file.
			fclose( file );
			// Report statistics.
			printf( "%u MPEG bytes transferred\n", b->mpeg_count );
			printf( "Video Stream stopped\n");
			prompt();
		}
		else
		{
			// Indicate there was a problem opening the MPEG data file.
			printf( "file open error\n" );
			SN_StopStream( b->handle);
			printf( "Video Stream stopped\n");
			prompt();
		}
	}
	else
	{ 
		printf("video thread exit\n");
	}
	return 0;
}


/*
 *  Create a WAV file header for up to 1 or 2-channel 16-bit PCM data.
 *  <len> in the number of bytes of PCM data.
 */
int wav_header_create(char *wavhdr, int len, int channels, int sample_rate)
{

	/* Do all the strings first, so we can use strcpy */
	strcpy(&wavhdr[WAV_RIFF_LIT], "RIFF");
	strcpy(&wavhdr[WAV_WAV_LIT], "WAVE");
	strcpy(&wavhdr[WAV_FMT_LIT], "fmt ");
	strcpy(&wavhdr[WAV_DATA_LIT], "data");
	*(int *) &wavhdr[WAV_LEN] = len + WAV_HEADERSIZE - 8;
	*(int *) &wavhdr[WAV_CHUNKLEN] = 0x10;
	*(short *) &wavhdr[WAV_FORMATID] = 1;
	*(short *) &wavhdr[WAV_NCHAN] = channels;
	*(int *) &wavhdr[WAV_SAMPLERATE] = sample_rate;

	/* For Stereo: 4 is number of channels * number of bytes per sample */
	/* For Mono: microphones only have one channel */
	/* The below configuration should handle both cases */

	*(int *) &wavhdr[WAV_BYTERATE] = sample_rate * channels * 2;
	*(short *) &wavhdr[WAV_BLOCKSIZE] = channels * 16 / 8;
	*(short *) &wavhdr[WAV_SAMPLESIZE] = 16;
	*(int *) &wavhdr[WAV_DATALEN] = len;

	return (0);
}


// audio work thread
void *aud_thread_func( void *arg )
{
	FILE					*file;								// MPEG data file.
	unsigned int	rlen;									// MPEG bytes transferred.
	time_t				starttime;						// Stream start time.
	BOARD					*b = (BOARD *)arg;		// Cache pointer to board object.
	char fname[260];
	char wavhdr[WAV_HEADERSIZE];

	// Select the function that will be used to move MPEG data between file and board.
	// S2250 encode only( MPEG_TO_FILE)
	int (*transfer)( struct BOARD *b, FILE *file );

	// Initialize stream statistics.
	b->mpeg_count = 0;		// No MPEG bytes have been transferred.
	time( &starttime );		// Log the start time.

	if( b->mode == MODE_RECORD_VIDONLY)
	{
		// end this thread, recording video only
		b->aud_thread_id = 0;
		return 0;
	}
	else if( b->mode == MODE_RECORD_AUDONLY)
	{
		transfer = aud_to_file;
		strcpy( fname, b->fname);
	}
	else if( b->mode == MODE_RECORD_SPLIT)
	{
		transfer = aud_to_file;
		strcpy( fname, b->fname);
		strcat( fname, ".wav");
	} else {
		transfer = NULL;
		printf("fatal error\n");
		return NULL;
	}

	if( ( b->mode == MODE_RECORD_AUDONLY) || ( b->mode == MODE_RECORD_SPLIT))
	{
		char *pWavhdr;
		int fp_fd; 
		void *fp_addr;
		// Attempt to open the WAV data file.  If successful ...
		if ( ( file = fopen( fname, "wb" ) ) != NULL )
		{
			// Report status.
			// write the header( use dummy 1024*1024 size for now)
			wav_header_create(wavhdr,1024 * 1024 ,
					  2, 48000);
			fwrite( wavhdr, 1, WAV_HEADERSIZE, file);
			printf( "Starting stream\n");
			if (b->mode != MODE_RECORD_SPLIT) {
				SN_StartStream( b->handle);
			}
			prompt();
			// Until stream termination is requested ...
			while ( !b->stop_rq )
			{
				//int SN_GetOneFrame(int boardnum, unsigned char *pBuf, SINT32 BufLen, TFrameInfo *pFI)
				rlen = transfer( b, file);
				// Allow other processes to execute if we are in non-blocking mode and no data was available.
				if ( ( rlen == 0 ) && G_polling)
				{
					// Sleep for approximately 10 milliseconds.
					struct timespec tv;
					tv.tv_sec = (time_t)0;
					tv.tv_nsec = (long)( 0.01 * 1e+9 );
					nanosleep( &tv, &tv );
				}
			}
			// Halt the stream.
			if( b->mode != MODE_IDLE)
			{
				if (b->mode != MODE_RECORD_SPLIT) {
					SN_StopStream( b->handle);
				}
			}
			// Close the audio data file.
			fclose( file );
			// reopen file and correct the length

			file = fopen(fname, "rb+");
			fp_fd = fileno(file);

			/* Synch RIFF header with # bytes recorded */
			fp_addr = mmap(0, WAV_HEADERSIZE,
				       PROT_READ | PROT_WRITE, MAP_SHARED, fp_fd, 0);

			if (!fp_addr) {
				printf( "Audio: mmap RIFF header failed!\n");
				return 0;
			}
	      
			pWavhdr = (char *) fp_addr;
			*(int *) &pWavhdr[WAV_DATALEN] = b->aud_count;
			munmap(fp_addr, WAV_HEADERSIZE);
			fclose( file);

			// Report statistics.
			printf( "%u audio bytes transferred\n", b->aud_count );
			printf( "Audio Stream stopped\n");
			prompt();
		}
		else
		{
			// Indicate there was a problem opening the WAV data file.
			printf( "file open error\n" );
			SN_StopStream( b->handle);
			printf( "Audio Stream stopped\n");
			prompt();
		}
	}
	return 0;
}

